
/* 
 * LCAUView.m -- Implementation file for the LCAUView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, or even that it has a use.
 *
 */

#import "LCAUView.h"
#import "drawingFuncs.h"
#import "nxlcau3t.h"
#import "nxardn3t.h"
#import <math.h> 
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <stdlib.h>
#import <string.h>

extern  int binrule[KK][KK][KK][KK];
extern char ascrule[KK][KK][KK][KK];
extern char auxrule[KK][KK][KK][KK];

int    drawBack=YES;
int    drawEvol=NO;
int    drawMosaic=NO;
int    drawPlane=NO;
int    drawShort=NO;
int    drawThird=NO;

int    drawview=0;

int    repPlEv=0;

extern NXColor colst[KK];
extern char colorwells[KK][6];

extern double blpr[KK];			/* probability from blocks	*/

extern int    co, dn;			/* connectivity, dimension  */
extern char   arule[DS], arr1[], arr2[], arr3[];

void     serms(double x, double y, int l, int m);
void     shortevolve(int m, int n);
void     tcolordot(double *p, double *c);
void     nxlcau3t();
double   tdist(int i, int j, int n);
void     wru(char *c);

@implementation LCAUView



void videocursor(p,i,j) int p, i, j; {}
void videoputc(c,i) char c; int i; {}
void videomode(i) int i; {}
void videopalette(i) int i; {}
void videoscroll(i,j,k,l,d,c) int i, j, k, l, d, c; {}
void videocattr(p,c,n,l) int p, c, n, l; {}


void videodot(i,j,l) int i, j, l; {
  if (l<3) NXSetColor(colst[l]); else NXSetColor(NX_COLORORANGE);
  switch (drawview) {
    case 0: PSrectfill((float)((2*AL-i+2*j)%(2*AL)),2.0*(float)i,1.25,1.25); break;
    case 1: PSrectfill(2.0*(float)i,398.0-2.0*(float)j,1.25,1.25); break;
    case 2: PSrectfill(8.0*(float)j+1.5,60.0-8.0*(float)i,4.75,4.75); break;
    case 3: PSrectfill(4.0*(float)j+2.0,48.0-4.0*(float)i,3.0,3.0); break;
    case 4: PSrectfill(2.0*(float)j,100.0-2.0*(float)i,1.25,1.25); break;
      case 5: PSrectfill(2.0*(float)j,2.0*(float)i,1.5,1.5); break;
      case 6: PSrectfill(10.0*(float)j,10.0*(float)i,9.0,9.0); break;
      case 7: PSrectfill(4.0*(float)j,4.0*(float)i,3.125,3.125); break;
      case 8: PSrectfill(150.0+(float)j,260.0-(float)i,1.0,1.0); break;
      case 9: PSrectfill(2.0*(float)j,173.0-2.0*(float)i,1.5,1.5); break;
      case 10: PSrectfill((float)j,(float)i,3.125,3.125); break;
      case 11: PSrectfill((float)j,(float)i,1.0,1.0); break;
      default: break;}
      
}

void tcolordot(p,c) double p[3], c[3]; {double s;
  s=1.0/(p[0]+p[1]+p[2]);
  PSsetcmykcolor(c[0],c[1],c[2],0.0);
  PSrectfill(200.0*(p[1]+0.5*p[2])*s,173.0-200.0*p[2]*s*0.866,1.5,1.5);
}

/*  m = line length; n = number of lines   */
/* Version for use with half-integer radii */
void shortevolve(m,n) int m, n; {
int i, j, k, l, o;
  savearry();
  asctobin();
  o=AL/2-m/2;
  for (j=0; j<n; j++) {
    k=j%2; l=j/2;
    for (i=0; i<m; i++) videodot(4*j,4*i-2*k,arr1[(YL+i+o+l)%YL]);
    onegen(YL);
    }
  renewarry();
}

/* index three dimensions */
// int inx3(i,j,k) int i, j, k; {
//	return (cu+i)%cu+ cu*((cu+j)%cu+ cu*((cu+k)%cu ));}

void drawsquare() {int i, j;
  for (i=0; i<128; i++) for (j=0; j<128; j++) videodot(j,i,arr1[i*128+j]);
}

void drawcube() {int i, j, k;
  for (i=0; i<32; i++) for (j=0; j<32; j++) for (k=0; k<16; k++) 
    videodot(j+32*(k/4),i+32*(k%4),arr1[ix3(i,j,k)]);
}

void manysquare(i,j) int i, j; {int k, l;
  for (k=0; k<128; k++) for (l=0; l<128; l++)  
    videodot(k+128*i,l+128*j,arr1[ix2(k,l)]);
}

void drawgrid() {int i;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<5; i++) {
  PSmoveto((double)(128.0*i),0.0); PSlineto((double)(128.0*i),512.0);}
  for (i=0; i<5; i++) {
  PSmoveto(0.0,(double)(128.0*i)); PSlineto(512.0,(double)(128.0*i));}
  PSstroke();
}

/* --------------------------- E V O L U T I O N --------------------------- */

/* evWindow decides which evolutionary window to display, based */
/* on the dimension of the automaton, which in turn depends on  */
/* how the neighborhood's volume can be factorized		*/

- evWindow:sender 
{  switch (dn) {
     case 1:  [ev1Window makeKeyAndOrderFront: sender];
	      [ev2Window performClose: sender];
	      [ev3Window performClose: sender];
	      break;
     case 2:  [ev2Window makeKeyAndOrderFront: sender];
	      [ev1Window performClose: sender];
	      [ev3Window performClose: sender];
	      break;
     default: [ev3Window makeKeyAndOrderFront: sender];
	      [ev1Window performClose: sender];
	      [ev2Window performClose: sender];
	      break;}
  return self;
}

- drawShEvol:sender {
    drawview=10;
    drawShort=YES;
    [self display];
    drawShort=NO;

return self;
}

- printShEvol:sender
{char ch[WNML];
    drawview=10;
    drawShort=YES;
    wru(ch);
    [ruleNumber setStringValue: ch];
   [keyWindow smartPrintPSCode:self];
    drawShort=NO;
return self;
}

- printLCAU:sender {
    if (keyWindow==evolWindow)    [self printEvolu:   sender];
    if (keyWindow==shEvolWindow)  [self printShEvol:  sender];
    if (keyWindow==thEvolWindow)  [self printThEvol:  sender];
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyLCAU:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printLCAU:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copyThEvol:sender {
NXRect zbounds;
id     pBoard;
  drawview=10;
  drawThird=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theThEvolView getBounds: &zbounds];
  [theThEvolView writePSCodeInside: &zbounds to: pBoard]; 
  drawThird=NO;
return self;
}

- copyShEvol:sender {
NXRect zbounds;
id     pBoard;
  drawview=10;
  drawShort=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theShEvolView getBounds: &zbounds];
  [theShEvolView writePSCodeInside: &zbounds to: pBoard]; 
  drawShort=NO;
return self;
}

- copyEvolu:sender {
NXRect zbounds;
id     pBoard;
  drawview=7;
  drawShort=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theShEvolView getBounds: &zbounds];
  [theShEvolView writePSCodeInside: &zbounds to: pBoard]; 
  drawShort=NO;
return self;
}

- copyLCAU:sender {
    if (keyWindow==evolWindow)    [self copyEvolu:  sender];
    if (keyWindow==shEvolWindow)  [self copyShEvol: sender];
    if (keyWindow==thEvolWindow)  [self copyThEvol: sender];
return self;
}

- drawEvolu:sender {
  drawview=0;
  drawEvol=YES;
  [self display];
  drawEvol=NO;
return self;
}

- printEvolu:sender {
  drawview=1;
  drawEvol=YES;
  [self printPSCode:self];
  drawEvol=NO;
return self;
}

- drawMosaic:sender {
  drawview=11;
  drawMosaic=YES;
  [self display];
  drawMosaic=NO;
return self;
}

- printMosaic:sender {
  drawview=11;
  drawMosaic=YES;
   [keyWindow smartPrintPSCode:self];
  drawMosaic=NO;
return self;
}

- drawPlEvol:sender {int i;
  repPlEv=[[sender selectedCell] tag];
  printf("drawPlEvol %d\n",repPlEv);
  drawview=7;
  drawPlane=YES;
  asctobin();
  if (repPlEv==0) [self display]; else
    for (i=0; i<repPlEv; i++) {
      [genEv setIntValue: i];
      onegen(AL);
      [self display];
      }
  [genEv setIntValue: repPlEv];
  drawPlane=NO;
return self;
}

- printPlEvol:sender {
    repPlEv=0;
    drawview=7;
    drawPlane=YES;
    [keyWindow smartPrintPSCode:self];
    drawPlane=NO;

return self;
}

/* -------------------------------- L C A U -------------------------------- */

- eaterRule:sender
{int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
  ascrule[i0][i1][i2][i3]=i0+'0';
  if ((i0+1)%KK==i1) ascrule[i0][i1][i2][i3]=i1+'0';
  if ((i0+1)%KK==i2) ascrule[i0][i1][i2][i3]=i2+'0';
  if ((i0+1)%KK==i3) ascrule[i0][i1][i2][i3]=i3+'0';
  }
return self;
}

/* -------------------------------- A R D N -------------------------------- */

/* -------------------------------- R I J N -------------------------------- */

/* -------------------------------- P R O B -------------------------------- */

/* -------------------------------- * * * * -------------------------------- */

/* - - - - - - - - - - -   L C A U V i e w   - - - - - - - - - - - - - - - - */


- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new LCAUView object.  First, an initFrame: message is sent 
 * to super to initialize LCAUView as a View.  Next, the LCAUView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
	NXPoint	spot;
	
	[super initFrame:frameRect];

	[self setOpaque:YES];
	crossCursor = 
		[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
	spot.x = spot.y = 7.0;
	[crossCursor setHotSpot:&spot];


  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];

	return self;

}

- setDelegate:anObject
/*
 * Sets the LCAUView's delegate instance variable to the supplied object.
*/
{
	delegate = anObject;
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the LCAUView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although LCAUView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
        char             c[WNML];
	unsigned int	 i, j, k, l, n;
	int		 ii, jj, nn, bas;
        double           en, p, q, r, s, t, dp, pee, cue;
	double		 aa[KK], bb[KK], cc[KK], cq[KK];
	double		 bern(), dbern(), tbern(), fbern();
	NXPoint		 *aPoint;
	NXRect		 plotRect;

  struct tm  *timeptr;
  time_t  secsnow;

int     i0, i1, ss;
	
	if (rects == NULL) return self;

	/* paint visible area white then draw axes */

	PSsetgray(NX_WHITE);
	if (drawBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

   wru(c);
   [ruleNumber setStringValue: c];

        if (drawEvol==YES) {
	  if ([keepTop state]) savearry();
	  evolve();
	  if ([keepTop state]) renewarry();
	  }

        if (drawShort==YES) {
  	  [self getBounds:&plotRect];
	  i=(int)(plotRect.size.width/4.0);
	  j=(int)(plotRect.size.height/4.0);
	  shortevolve(i,j);
	  }

        if (drawThird==YES) {
  	  [self getBounds:&plotRect];
	  i=(int)(plotRect.size.width/10.0);
	  j=(int)(plotRect.size.height/10.0);
	  shortevolve(i,j);
	  }

        if (drawPlane==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "Moore (3,1/2) Plane Evolution"];
	drawsquare();
	break;
     case 3: 
	[window setTitle: "3-D cube corner Solid Evolution"];
	drawcube();
	drawgrid();
	break;
     case 5: 
	[window setTitle: "von Neumann (3,1) Plane Evolution"];
	drawsquare();
	break;
     case 6: 
	[window setTitle: "3-D tetrahedral center Solid Evolution"];
	drawcube();
	drawgrid();
	break;
     default: [window setTitle: "LCAU3T Plane Evolution"];break;}
	}

        if (drawMosaic==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "Moore (3,1/2) manysquare Plane Evolution"];
	for (i=0; i<4; i++) for (j=0; j<4; j++) {
	  manysquare(i,j);
	  onegen(AL);
	  [window flushWindow];
	  }
	drawgrid();
	break;
     case 5: 
	[window setTitle: "von Neumann (3,1) manysquare Plane Evolution"];
	for (i=0; i<4; i++) for (j=0; j<4; j++) {
	  manysquare(i,j);
	  onegen(AL);
	  [window flushWindow];
	  }
	drawgrid();
	break;
     default: [window setTitle: "LCAU3T Plane Evolution"];break;}
	}

//        if (drawLine==YES) {
//        for (j=0; j<8; j++) for (i=0; i<40; i++) videodot(j,i,arr1[i+40*j]);
//	}

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the LCAUView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo:width :height];
	if (drawview==4||drawview==7||drawview==10) 
		{[self setDrawOrigin:0 :0]; return self;}
	else 
		[self setDrawOrigin: -floor(width/2): -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the LCAUView.  The LCAUView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the LCAUView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the LCAUView, the LCAUView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect;

         double  x, y, ex, wy, eks, wye;
	 int k;

	[crossCursor set];
   	[self getBounds:&plotRect];

	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
//                ex=aPoint.x;
//                wy=aPoint.y;
				PSsetgray(NX_BLACK);
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {looping = NO;}	
   	} while (looping && 
	(theEvent=[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];


	}
	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

- read:(NXTypedStream *)stream
/*
 * Unarchives the LCAUView.  Initializes four of the LCAUView's instance 
 * variables to the values stored in the stream. 
 */
{
	[super read:stream];
	return self;
}

- write:(NXTypedStream *)stream
/*
 * Archives the LCAUView by writing its important instance variables to the 
 * stream. 
 */
{
	[super write:stream];
	return self;
}

- awake
/*
 * Finishes initializing a LCAUView that has just been unarchived (see the 
 * read: method).
 */
{
	loadPSProcedures();
	return [super awake];
}


- (const char *)inspectorName
{
    return "LCAUViewInspector";
}

@end

/* end LCAUView.m */
