
/* nxcycl3t.c						*/
/* program for LCA3t option 'd'				*/
/* display de Bruijn diagram - 1st, 2nd, & 3rd gen	*/
/* Harold V. McIntosh, 20 February 1988			*/
/* 21 February 1988 - add j0=i0; to Pass 4's		*/
/* 26 September 1989 - show evolution of periodic conf	*/
/* 2 October 1989 - cycl31.c incorporated into rijn31.c	*/
/* 20 November 1989 - correct loop in kwait		*/
/* 22 June 1994 - adapt to NeXT				*/
/* 21 June 1995 - separated cycle from de Bruijn	*/

/*    Copyright (C) 1987		*/
/*    Copyright (C) 1988		*/
/*    Copyright (C) 1989		*/
/*    Copyright (C) 1994		*/
/*    Copyright (C) 1995		*/
/*    Harold V. McIntosh		*/

# include <math.h>
# include "nxlcau3t.h"
# include "nxcycl3t.h"
# import  <dpsclient/psops.h>

extern char arr1[AL];

int    cylink[XCY];		/* links in the cycle diagram		*/
int    cynanc[XCY][21];		/* no of ancestors in the cycle diagram	*/
int    cyhanc[XCY];		/* ancestor histogram for cycles	*/
int    cyhigh[XCY];		/* level of cycle diagram nodes		*/
int    cymass[XCY];		/* number of nodes in node's own tree	*/
int    cyroot[XCY];		/* root for each node in cycle diagram	*/
int    cycomp[XCY];		/* component for node in cycle diagram	*/
int    cyarry[XCY];		/* scratch array for cycle diagrams	*/
double ccntr[XCY][2];		/* coordinates of cycle diagram nodes	*/

int  cc, cy; 			/* number of points in cycle array 	*/
int  xhigh, nhigh;		/* maximum height, minimum height  	*/
int  xroot, nroot;		/* maximum cycle, minimum cycle    	*/
int  cynum;			/* number of cycles			*/

/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */

/* CYCLE3t.C					 */

/* Pass 1 makes arry[i] equal to the successor of i */
/* the configuration is supposed to be cyclic	    */

void ylpass1(cy,cp,arry) int cy, cp, *arry; {int i, j, x;
asctobin();
for (j=0; j<cp; j++) {
  x=j; for (i=0; i<cy; i++) {arr1[cy-i-1]=x%KK; x/=KK;}
  onegen(cy);
  x=0; for (i=0; i<cy; i++) x=KK*x+arr1[i]; arry[j]=x;
  }
}

/* zylpass 1 makes arry[i] equal to the successor of i 		*/
/* the configuration is supposed to have fixed boundary cells 	*/
/* cy = length of configuration					*/
/* cp = KK^cy = number of configurations of length cy		*/
/* bl = state of left boundary cell				*/
/* br = state of right boundary cell				*/

void zylpass1(cy,cp,bl,br,arry) int cy, cp, bl, br; unsigned int *arry; {
int i, j, x;
asctobin();
for (j=0; j<cp; j++) {
  arr1[0]=bl;
  x=j; for (i=1; i<cy+1; i++) {arr1[cy-i-1]=x%KK; x/=KK;}
  arr1[cy+1]=br;
  onegen(cy+2);
  x=0; for (i=1; i<cy+1; i++) x=KK*x+arr1[i]; arry[j]=x;
  }
// printf("ylpass1 done\n");
}

/* Pass 2 flags all links with an incoming arrow */

void ylpass2(cp,arry) int cp; unsigned int *arry; {int j, m, x;
videocursor(0,2,0); printf(" Pass2\015");
do {
  m=0;
  for (j=0; j<cp; j++) arry[j]|=0x8000;
  for (j=0; j<cp; j++) {x=arry[j];
   if (x!=0xFFFF) arry[x&0x7FFF]&=0x7FFF;}
  for (j=0; j<cp; j++) {
   if ((arry[j]>0x7FFF) && (arry[j]!=0xFFFF)) {arry[j]=0xFFFF; m=1;}}
  } while (m!=0);
}

/* print the link */

prf(cy,j,x) int j, x; {int i, y;
  y=j; for (i=0; i<cy; i++) {arr1[i]=y%KK; y/=KK;}
  for (i=0; i<cy; i++) printf("%1d",arr1[i]);
  printf("-");
  y=x; for (i=0; i<cy; i++) {arr1[i]=y%KK; y/=KK;}
  for (i=0; i<cy; i++) printf("%1d",arr1[i]);
  printf("  ");
}

/* show sample of the cycles */
void ylpass5(cy,cp,arry) int cy, cp; unsigned int *arry; {}

/* edit the Pass 4 and Pass 5 cycle displays 	*/
/*   arry = linkage list			*/
/*   cc   = number of cycles			*/
/*   k    = label style (self, hight, root, &c)	*/
/*   l    = node to emphasize by heavy circle	*/
/*   cn   = reference node w/r/t m		*/
/*   m    = option for suppressing nodes	*/
void edcyl(arry,cc,k,l,cn,m) int *arry, cc, k, l, cn, m; {
int    i, j;
char   s[10];
double x, y, ex, wy, eks, wye, dx, dy, r, t;
  if (cc>XCY) return;
  r=10.3;
  for (i=0; i<cc; i++) {

	j=arry[i];
	if (m==0 && cyroot[i]!=cyroot[cn]) continue;	/* tree      */
	if (m==1 && cyhigh[i]>0) continue;		/* roots     */
	if (m==2 && cycomp[i]!=cycomp[cn]) continue;	/* component */
	ex=ccntr[i][0];
	wy=ccntr[i][1];
	PSmoveto(ex,wy);
	x=ccntr[j][0];
	y=ccntr[j][1];
	dx=x-ex;
	dy=y-wy;
	t=atan2(dy,dx);
	eks=x-r*cos(t);
	wye=y-r*sin(t);
	PSlineto(eks,wye);
	PSlineto(eks-0.1*dx+0.05*dy,wye-0.1*dy-0.05*dx);
	PSlineto(eks-0.1*dx-0.05*dy,wye-0.1*dy+0.05*dx);
	PSlineto(eks,wye);
	PSmoveto(x+r,y);
	PSarc(x, y, r, 0.0, 360.0);


   ex=ccntr[i][0];
   wy=ccntr[i][1];
   PSmoveto(ex-10.0,wy+5.0);
   switch (k) {
     case 0: sprintf(s,"%4d",i); break;
     case 1: sprintf(s,"%4d",cyroot[i]); break;
     case 2: sprintf(s,"%4d",cyhigh[i]); break;
     default: sprintf(s," "); break; }
   PSshow(s);

   }

if (l<XCY) {
	PSstroke();
	x=ccntr[l][0];
	y=ccntr[l][1];
	PSsetlinewidth(3.0);
	PSmoveto(x+r,y);
	PSarc(x, y, 0.8*r, 0.0, 360.0);
        }
PSstroke();
PSsetlinewidth(0.0);
}

void nicetree(root,x0,y0,r,th,dt) int root; double x0, y0, r, th, dt; {
int i, j; double eye, jay;
// printf("hello from nicetree %3d\n",root);
  ccntr[root][0]=x0;
  ccntr[root][1]=y0;
// printf("x=%6.2f, y=%6.2f\n",x0,y0);
  if (cynanc[root][0]==0) return;
  jay=dt*((double)(cynanc[root][0])/2.0);
  for (i=2; i<cynanc[root][1]; i++) {
	j=cynanc[root][i];
// printf("i=%3d, j=%3d\n",i,j);
	if (cyroot[j]==j) continue;
	eye=(double)i;
	nicetree(j,
		 x0+r*cos(th+eye*jay),
		 y0+r*sin(th+eye*jay),
		 r,
		 th+eye*jay,
		 0.5*dt);
  }
}

/* generate the cycle linkage list */
void gecyl(arry,cy,bl,br,l) int *arry, cy, bl, br, l; {
int i, cp;
if (cy>11) return;
for (i=0, cp=1; i<cy; i++) cp*=KK;
  if (l==0) ylpass1(cy,cp,arry);
  if (l==1) zylpass1(cy,cp,bl,br,arry);
}

/* find index of nearest center to x, y (cycles) */
int nearby(x,y) double x, y; {int i, k; double ex, wy, r, p;
k=0; p=500000.0;
for (i=0; i<cc; i++) {
  ex=x-ccntr[i][0];
  wy=y-ccntr[i][1];
  r=ex*ex+wy*wy;
  if (r<p) {p=r; k=i;}
  }
return k;
}

/* count the number of ancestors     */
/* and keep a list, up to 19 of them */
void geanc(ance,link,cp) int ance[][21], *link, cp; {
int i, j, k;
for (i=0; i<cp; i++) {ance[i][0]=0; ance[i][1]=2;}
for (i=0; i<cp; i++) {
  j=link[i];
  ance[j][0]++;
  k=ance[j][1];
  if (k<=21) {ance[j][k]=i; ance[j][1]++;}
  }
}

/* make a histogram of ancestor counts */
void hianc(anhi,ance,cp) int *anhi, *ance, cp; {
int i;
for (i=0; i<cp; i++) anhi[i]=0;
for (i=0; i<cp; i++) anhi[ance[i]]++;
}

/* count the number of occurrences of a given root */
void hiroo(rohi,root,cp) int *rohi, *root, cp; {
int i;
for (i=0; i<cp; i++) rohi[i]=0;
for (i=0; i<cp; i++) rohi[root[i]]++;
}

/* find the root to which each node is attached     */
/* find the component to which each node belongs    */
/* each node on a cycle is the root of its own tree */
void geroo(root,comp,link,cp) int *root, *comp, *link, cp; {
int i, j, k, l;
  cynum=0;
  xroot=0;
  nroot=XCY;
  for (i=0; i<cp; i++) cyarry[i]=0;
  for (i=0; i<cp; i++) {
    j=i;
    if (cyarry[j]==2) continue;
    while (cyarry[j]==0) {cyarry[j]=1; j=link[j];}
    if (cyarry[j]==1) {
      k=j;
      l=0;
      cynum++;
      while (cyarry[k]==1) {cyarry[k]=2; root[k]=k; comp[k]=j; k=link[k]; l++;}
      }
    if (l>xroot) xroot=l;
    if (l<nroot) nroot=l;
    if (cyarry[j]==2) j=root[j];
    for (k=0; k<cp; k++) 
	if (cyarry[k]==1) {root[k]=j; comp[k]=comp[j]; cyarry[k]=2;}
    }
}

/* find the height of each node, assuming that the roots are known */
void gehei(high,root,link,cp) int *high, *root, *link, cp; {
int i, j, k, l;
  xhigh=0;
  nhigh=XCY;
  for (i=0; i<cp; i++) high[i]=XCY;
  for (i=0; i<cp; i++) high[root[i]]=0;
  for (i=0; i<cp; i++) {
    for (l=0, j=0; j<cp; j++) {
      if (j==root[j]) continue;
      k=high[link[j]]; 
      if (k<XCY) {high[j]=k+1; l=1;}
      }
    if (l==0) break;
    }
  xhigh=0;
  nhigh=XCY;
  for (i=0; i<cp; i++) {
    j=high[i];
    if (j==XCY) continue;
    if (j>xhigh) xhigh=j;
    if (j<nhigh) nhigh=j;
  }
}

/* move the in-linked nodes into close proximity */
void ingather(clst,arry,k,n) double clst[][2]; int *arry, k, n; {
int i;
double x, xx, y, yy;
  x = clst[k][0];
  y = clst[k][1];
  for (i=0; i<n; i++) if (arry[i]==k) {
    xx = clst[i][0];
    yy = clst[i][1];
    clst[i][0]=0.8*x+0.2*xx;
    clst[i][1]=0.8*y+0.2*yy;
    }
}

/* move the out-linked node into close proximity */
void outgather(clst,arry,k) double clst[][2]; int  *arry, k; {
int j;
double x, xx, y, yy;
  j=arry[k];
  x = clst[k][0];
  y = clst[k][1];
  xx = clst[j][0];
  yy = clst[j][1];
  clst[j][0]=0.8*x+0.2*xx;
  clst[j][1]=0.8*y+0.2*yy;
}

/* end nxcycl3t.c */
